//
//  ContentMainTabFreeViewModel.swift
//  SodaLive
//
//  Created by klaus on 2/22/25.
//

import Foundation
import Combine

final class ContentMainTabFreeViewModel: ObservableObject {
    private let repository = ContentMainTabFreeRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    @Published var isLoading = false
    
    @Published var bannerList: [GetAudioContentBannerResponse] = []
    @Published var introduceCreator: GetContentCurationResponse? = nil
    @Published var recommendSeriesList: [GetRecommendSeriesListResponse] = []
    @Published var themeList: [String] = []
    @Published var newFreeContentList: [GetAudioContentMainItem] = []
    @Published var creatorList: [ContentCreatorResponse] = []
    @Published var playCountRankContentList: [GetAudioContentRankingItem] = []
    @Published var curationList: [GetContentCurationResponse] = []
    
    func fetchData() {
        isLoading = true
        repository.getContentMainFree()
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetContentMainTabFreeResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.bannerList = data.contentBannerList
                        self.introduceCreator = data.introduceCreator
                        self.recommendSeriesList = data.recommendSeriesList
                        self.newFreeContentList = data.newFreeContentList
                        self.creatorList = data.creatorList
                        self.playCountRankContentList = data.playCountRankContentList
                        self.curationList = data.curationList
                        
                        self.themeList.removeAll()
                        self.themeList.append("전체")
                        self.themeList.append(contentsOf: data.themeList)
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getNewContentOfTheme(theme: String) {
        isLoading = true
        repository.getNewContentOfTheme(theme: theme)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetAudioContentMainItem]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.newFreeContentList = data
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getPopularContentByCreator(creatorId: Int) {
        isLoading = true
        repository.getPopularContentByCreator(creatorId: creatorId)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetAudioContentRankingItem]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.playCountRankContentList = data
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
}
