//
//  CreatorCommunityCommentListViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/12/20.
//

import Foundation
import Combine

final class CreatorCommunityCommentListViewModel: ObservableObject {
    private let repository = CreatorCommunityRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var isLoading = false
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    
    @Published var comment = ""
    @Published var totalCommentCount = 0
    @Published var commentList = [GetCommunityPostCommentListItem]()
    
    var postId = 0
    var page = 1
    var isLast = false
    private let pageSize = 10
    
    func getCommentList() {
        if (!isLast && !isLoading) {
            repository
                .getCommunityPostCommentList(postId: postId, page: page, size: pageSize)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponse<GetCommunityPostCommentListResponse>.self, from: responseData)
                        
                        if let data = decoded.data, decoded.success {
                            if page == 1 {
                                commentList.removeAll()
                            }
                            
                            if !data.items.isEmpty {
                                page += 1
                                self.totalCommentCount = data.totalCount
                                self.commentList.append(contentsOf: data.items)
                            } else {
                                isLast = true
                            }
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                    
                    self.isLoading = false
                }
                .store(in: &subscription)
        }
    }
    
    func createCommunityPostComment(parentId: Int? = nil) {
        if !isLoading {
            isLoading = true
            
            repository.createCommunityPostComment(comment: comment, postId: postId, parentId: parentId)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    let responseData = response.data
                    self.isLoading = false
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                        
                        if decoded.success {
                            DispatchQueue.main.async {
                                self.comment = ""
                                self.page = 1
                                self.isLast = false
                                self.getCommentList()
                            }
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                }
                .store(in: &subscription)
        }
    }
    
    func modifyComment(
        commentId: Int,
        postId: Int,
        comment: String? = nil,
        isActive: Bool? = nil
    ) {
        if comment == nil && isActive == nil {
            errorMessage = "변경사항이 없습니다."
            isShowPopup = true
            return
        }
        
        if let comment = comment, comment.trimmingCharacters(in: .whitespaces).isEmpty {
            errorMessage = "내용을 입력하세요."
            isShowPopup = true
            return
        }
        
        isLoading = true
        
        var request = ModifyCommunityPostCommentRequest(commentId: commentId)
        
        if let comment = comment {
            request.comment = comment
        }
        
        if let isActive = isActive {
            request.isActive = isActive
        }
        
        repository.modifyComment(request: request)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                    
                    if decoded.success {
                        self.comment = ""
                        self.page = 1
                        self.isLast = false
                        
                        self.commentList.removeAll()
                        DispatchQueue.main.asyncAfter(deadline: .now() + 0.2) {
                            self.getCommentList()
                        }
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.isLoading = false
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
}
