//
//  CreatorCommunityAllViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/12/19.
//

import Foundation

import Moya
import Combine

class CreatorCommunityAllViewModel: ObservableObject {
    private let repository = CreatorCommunityRepository()
    private let reportRepository = ReportRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var isLoading = false
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    @Published private(set) var communityPostList = [GetCommunityPostListResponse]()
    
    @Published var postId = 0
    @Published var postPrice = 0
    @Published var postIndex = -1
    
    @Published var isShowCommentListView = false {
        didSet {
            if !isShowCommentListView {
                postId = 0
            }
        }
    }
    
    @Published var isShowReportMenu = false
    @Published var isShowReportView = false {
        didSet {
            if !isShowReportView {
                postId = 0
            }
        }
    }
    @Published var isShowDeleteConfirm = false {
        didSet {
            if !isShowDeleteConfirm {
                postId = 0
            }
        }
    }
    
    @Published var isShowPostPurchaseView = false {
        didSet {
            if !isShowPostPurchaseView {
                postId = 0
                postPrice = 0
                postIndex = -1
            }
        }
    }
    
    var creatorId = 0
    
    var page = 1
    var isLast = false
    private var pageSize = 10
    
    func getCommunityPostList() {
        if !isLoading && !isLast {
            isLoading = true
            
            repository
                .getCommunityPostList(creatorId: creatorId, page: page, size: pageSize)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponse<[GetCommunityPostListResponse]>.self, from: responseData)
                        
                        if let data = decoded.data, decoded.success {
                            if data.count > 0 {
                                if page == 1 {
                                    self.communityPostList.removeAll()
                                }
                                
                                self.communityPostList.append(contentsOf: data)
                                self.page += 1
                                self.pageSize = 10
                            } else {
                                isLast = true
                            }
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                    
                    self.isLoading = false
                }
                .store(in: &subscription)
        }
    }
    
    func communityPostLike(postId: Int) {
        repository.communityPostLike(postId: postId)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { _ in
            }
            .store(in: &subscription)
    }
    
    func createCommunityPostComment(comment: String, postId: Int, parentId: Int? = nil) {
        isLoading = true
        
        repository.createCommunityPostComment(comment: comment, postId: postId, parentId: parentId)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                    
                    if decoded.success {
                        DispatchQueue.main.async {
                            self.pageSize *= self.page
                            self.page = 1
                            self.isLast = false
                            self.getCommunityPostList()
                        }
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func report(reason: String) {
        isLoading = true
        
        let request = ReportRequest(type: .COMMUNITY_POST, reason: reason, communityPostId: postId)
        reportRepository.report(request: request)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                    
                    if let message = decoded.message {
                        self.errorMessage = message
                    } else {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    }
                    
                    self.isShowPopup = true
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
    
    func deleteCommunityPost() {
        isLoading = true
        
        let request = ModifyCommunityPostRequest(creatorCommunityId: postId, isActive: false)
        var multipartData = [MultipartFormData]()
        
        let encoder = JSONEncoder()
        encoder.outputFormatting = .withoutEscapingSlashes
        let jsonData = try? encoder.encode(request)
        
        if let jsonData = jsonData {
            multipartData.append(MultipartFormData(provider: .data(jsonData), name: "request"))
            
            repository.modifyCommunityPost(parameters: multipartData)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    self.isLoading = false
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                        
                        if decoded.success {
                            self.errorMessage = "삭제되었습니다"
                            self.isShowPopup = true
                            
                            self.page = 1
                            self.isLast = false
                            self.getCommunityPostList()
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                }
                .store(in: &subscription)
        } else {
            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
            self.isShowPopup = true
            self.isLoading = false
        }
    }
    
    func purchaseCommunityPost() {
        let postId = postId
        let postIndex = postIndex
        
        if !isLoading {
            isLoading = true
            
            repository
                .purchaseCommunityPost(postId: postId)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponse<GetCommunityPostListResponse>.self, from: responseData)
                        
                        if let data = decoded.data, decoded.success {
                            if postIndex >= 0 {
                                communityPostList[postIndex] = data
                            }
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                    
                    self.isLoading = false
                }
                .store(in: &subscription)
        }
    }
}
