//
//  CreatorCommunityModifyViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/12/21.
//

import UIKit
import Moya
import Combine

final class CreatorCommunityModifyViewModel: ObservableObject {
    private let repository = CreatorCommunityRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var isLoading = false
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    
    @Published var content = ""
    @Published var isAdult = false
    @Published var isAvailableComment = true
    @Published var postImage: UIImage? = nil
    @Published var postImageUrl: String? = nil
    @Published private(set) var communityPost: GetCommunityPostListResponse?
    
    var placeholder = "내용을 입력하세요"
    var postId = 0
    
    func getCommunityPostDetail(onFailure: (() -> Void)? = nil) {
        communityPost = nil
        isLoading = true
        
        repository.getCommunityPostDetail(postId: postId)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetCommunityPostListResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.communityPost = data
                        
                        self.content = data.content
                        self.isAdult = data.isAdult
                        self.isAvailableComment = data.isCommentAvailable
                        self.postImageUrl = data.imageUrl
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                        if let onFailure = onFailure {
                            onFailure()
                        }
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                    if let onFailure = onFailure {
                        onFailure()
                    }
                }
            }
            .store(in: &subscription)
        
    }
    
    func modifyCommunityPost(onSuccess: @escaping () -> Void) {
        if !isLoading && validateData() {
            isLoading = true
            
            let request = ModifyCommunityPostRequest(
                creatorCommunityId: postId,
                content: communityPost!.content != content ? content : nil,
                isCommentAvailable: communityPost!.isCommentAvailable != isAvailableComment ? isAvailableComment : nil,
                isAdult: communityPost!.isAdult != isAdult ? isAdult : nil
            )
            var multipartData = [MultipartFormData]()
            
            let encoder = JSONEncoder()
            encoder.outputFormatting = .withoutEscapingSlashes
            let jsonData = try? encoder.encode(request)
            
            if let jsonData = jsonData {
                if let postImage = postImage, let imageData = postImage.jpegData(compressionQuality: 0.8) {
                    multipartData.append(
                        MultipartFormData(
                            provider: .data(imageData),
                            name: "postImage",
                            fileName: "\(UUID().uuidString)_\(Date().timeIntervalSince1970 * 1000).jpg",
                            mimeType: "image/*")
                    )
                }
                
                multipartData.append(MultipartFormData(provider: .data(jsonData), name: "request"))
                
                repository
                    .modifyCommunityPost(parameters: multipartData)
                    .sink { result in
                        switch result {
                        case .finished:
                            DEBUG_LOG("finish")
                        case .failure(let error):
                            ERROR_LOG(error.localizedDescription)
                        }
                    } receiveValue: { [unowned self] response in
                        self.isLoading = false
                        let responseData = response.data
                        
                        do {
                            let jsonDecoder = JSONDecoder()
                            let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                            
                            if decoded.success {
                                self.errorMessage = "게시물이 수정되었습니다."
                                self.isShowPopup = true
                                
                                DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
                                    onSuccess()
                                }
                            } else {
                                if let message = decoded.message {
                                    self.errorMessage = message
                                } else {
                                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                                }
                                
                                self.isShowPopup = true
                            }
                        } catch {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            self.isShowPopup = true
                        }
                    }
                    .store(in: &subscription)
            } else {
                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                self.isShowPopup = true
                self.isLoading = false
            }
        }
    }
    
    private func validateData() -> Bool {
        if content.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty || content.count < 5 {
            errorMessage = "내용을 5자 이상 입력해 주세요."
            isShowPopup = true
            return false
        }
        
        return true
    }
}
