//
//  CanPgPaymentViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/08/11.
//

import Foundation
import Combine

import Bootpay

enum PaymentMethod: String {
    case card = "카드"
    case bank = "계좌이체"
    case phone = "휴대폰"
    case kakaopay = "카카오페이"
}

final class CanPgPaymentViewModel: ObservableObject {
    
    private let repository = CanRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var isTermsAgree = false
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    @Published var isLoading = false
    
    @Published var isShowPaymentView = false
    @Published var paymentMethod: PaymentMethod? = nil
    
    let payload = Payload()
    
    func chargeCan(canId: Int, onSuccess: @escaping () -> Void) {
        isLoading = true
        repository.pgChargeCan(canId: canId)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<CanChargeResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        payload.applicationId = BOOTPAY_APP_HECTO_ID
                        payload.pg = paymentMethod == .kakaopay ? "카카오" : "세틀뱅크"
                        payload.orderId = "\(data.chargeId)"
                        payload.method = paymentMethod!.rawValue
                        
                        var username = UserDefaults.string(forKey: .nickname)
                        
                        if username.count > 10 {
                            username = "\(username.prefix(6))..."
                        }
                        
                        let bootUser = BootUser()
                        bootUser.userId = "\(UserDefaults.int(forKey: .userId))"
                        bootUser.username = UserDefaults.string(forKey: .nickname)
                        
                        payload.user = bootUser
                        
                        onSuccess()
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
    
    func verifyPayment(_ data: [String: Any], onSuccess: @escaping () -> Void) {
        isLoading = true
        
        let _data = data["data"] as? [String: Any]
        
        if let data = _data {
            let receiptId = data["receipt_id"] as! String
            let orderId = data["order_id"] as! String
            
            repository.pgVerifyHecto(receiptId: receiptId, orderId: orderId)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    self.isLoading = false
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                        
                        if decoded.success {
                            self.errorMessage = "캔이 충전되었습니다"
                            self.isShowPopup = true
                            
                            onSuccess()
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                }
                .store(in: &subscription)
        } else {
            isLoading = false
            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
            isShowPopup = true
        }
    }
}
